@@ -9,6 +9,6 @@ class RE_Matcher;
 namespace binpac
 	{
 
-std::vector<zeek::RE_Matcher*>* uncompiled_re_matchers = 0;
+std::vector<zeek::RE_Matcher*>* uncompiled_re_matchers = nullptr;
 
 	}
