@@ -7,28 +7,28 @@ class FlowDecl : public AnalyzerDecl
 	{
 public:
 	FlowDecl(ID* flow_id, ParamList* params, AnalyzerElementList* elemlist);
-	~FlowDecl();
+	~FlowDecl() override;
 
-	void Prepare();
+	void Prepare() override;
 
 	void set_conn_decl(ConnDecl* c) { conn_decl_ = c; }
 
 	static ParameterizedType* flow_buffer_type();
 
 protected:
-	void AddBaseClass(vector<string>* base_classes) const;
+	void AddBaseClass(vector<string>* base_classes) const override;
 
-	void GenInitCode(Output* out_cc);
-	void GenCleanUpCode(Output* out_cc);
-	void GenProcessFunc(Output* out_h, Output* out_cc);
-	void GenEOFFunc(Output* out_h, Output* out_cc);
-	void GenGapFunc(Output* out_h, Output* out_cc);
+	void GenInitCode(Output* out_cc) override;
+	void GenCleanUpCode(Output* out_cc) override;
+	void GenProcessFunc(Output* out_h, Output* out_cc) override;
+	void GenEOFFunc(Output* out_h, Output* out_cc) override;
+	void GenGapFunc(Output* out_h, Output* out_cc) override;
 
-	void GenPubDecls(Output* out_h, Output* out_cc);
-	void GenPrivDecls(Output* out_h, Output* out_cc);
+	void GenPubDecls(Output* out_h, Output* out_cc) override;
+	void GenPrivDecls(Output* out_h, Output* out_cc) override;
 
-	void ProcessFlowElement(AnalyzerFlow* flow_elem);
-	void ProcessDataUnitElement(AnalyzerDataUnit* dataunit_elem);
+	void ProcessFlowElement(AnalyzerFlow* flow_elem) override;
+	void ProcessDataUnitElement(AnalyzerDataUnit* dataunit_elem) override;
 
 private:
 	void GenNewDataUnit(Output* out_cc);
