@@ -23,11 +23,11 @@ sys.path.insert(0, os.path.abspath('scripts'))
 # -- Project information -----------------------------------------------------
 
 project = u'Spicy'
-copyright = u'2020 by the Zeek Project'
+copyright = u'2023 by the Zeek Project'
 author = u'Zeek Project'
 
 version = open('../VERSION').readline()
-release = "1.8.2"  # most recent release version
+release = "1.9.0"  # most recent release version
 
 # -- General configuration ---------------------------------------------------
 
@@ -59,6 +59,8 @@ extlinks = {
     "issue": ("https://github.com/zeek/spicy/issues/%s", "#%s"),
     "pr":    ("https://github.com/zeek/spicy/pulls/%s", "#%s"),
 
+    "zeek":  ("https://docs.zeek.org/en/master/%s", "%s"),
+
     # Links to binary builds.
     "package-dev-tgz": ("https://api.cirrus-ci.com/v1/artifact/github/zeek/spicy/%s/packages/build/spicy-dev.tar.gz", "%s"),
     "package-dev-rpm": ("https://api.cirrus-ci.com/v1/artifact/github/zeek/spicy/%s/packages/spicy-dev.rpm", "%s"),
@@ -83,7 +85,10 @@ html_theme_options = {
 linkcheck_ignore = [
     r'https://api.cirrus-ci.com/v1/artifact/github/zeek/spicy/.*',
     r'http://download.zeek.org',
-    r'https://download.zeek.org']
+    r'https://download.zeek.org',
+    # Certificate potentially cannot be validated.
+    r'https://www.icir.org/hilti',
+]
 
 read_the_docs_build = os.environ.get('READTHEDOCS', None) == 'True'
 if read_the_docs_build:
