@@ -55,16 +55,16 @@ static const std::string debug_stream_fibers = "fibers";
 // Wrapper similar to HILTI_RT_DEBUG that adds the current fiber to the message.
 #define HILTI_RT_FIBER_DEBUG(tag, msg)                                                                                 \
     {                                                                                                                  \
-        if ( ::hilti::rt::detail::unsafeGlobalState()->debug_logger &&                                                       \
-             ::hilti::rt::detail::unsafeGlobalState()->debug_logger->isEnabled(debug_stream_fibers) )                        \
+        if ( ::hilti::rt::detail::unsafeGlobalState()->debug_logger &&                                                 \
+             ::hilti::rt::detail::unsafeGlobalState()->debug_logger->isEnabled(debug_stream_fibers) )                  \
             ::hilti::rt::debug::detail::print(debug_stream_fibers,                                                     \
                                               fmt("[%s/%s] %s", *context::detail::get()->fiber.current, tag, msg));    \
     }
 
 #define HILTI_RT_FIBER_DEBUG_NO_CONTEXT(tag, msg)                                                                      \
     {                                                                                                                  \
-        if ( ::hilti::rt::detail::unsafeGlobalState()->debug_logger &&                                                       \
-             ::hilti::rt::detail::unsafeGlobalState()->debug_logger->isEnabled(debug_stream_fibers) )                        \
+        if ( ::hilti::rt::detail::unsafeGlobalState()->debug_logger &&                                                 \
+             ::hilti::rt::detail::unsafeGlobalState()->debug_logger->isEnabled(debug_stream_fibers) )                  \
             ::hilti::rt::debug::detail::print(debug_stream_fibers, fmt("[none/%s] %s", tag, msg));                     \
     }
 
@@ -173,8 +173,8 @@ detail::Fiber::Fiber(Type type) : _type(type), _fiber(std::make_unique<::Fiber>(
         }
 
         case Type::SwitchTrampoline:
-            if ( ! ::fiber_alloc(_fiber.get(), configuration::detail::unsafeGet().fiber_individual_stack_size, fiber_bottom_abort,
-                                 this, FiberGuardFlags) )
+            if ( ! ::fiber_alloc(_fiber.get(), configuration::detail::unsafeGet().fiber_individual_stack_size,
+                                 fiber_bottom_abort, this, FiberGuardFlags) )
                 internalError("could not allocate individual-stack fiber");
 
 #ifdef HILTI_HAVE_ASAN
@@ -195,8 +195,8 @@ detail::Fiber::Fiber(Type type) : _type(type), _fiber(std::make_unique<::Fiber>(
         }
 
         case Type::IndividualStack: {
-            if ( ! ::fiber_alloc(_fiber.get(), configuration::detail::unsafeGet().fiber_individual_stack_size, fiber_bottom_abort,
-                                 this, FiberGuardFlags) )
+            if ( ! ::fiber_alloc(_fiber.get(), configuration::detail::unsafeGet().fiber_individual_stack_size,
+                                 fiber_bottom_abort, this, FiberGuardFlags) )
                 internalError("could not allocate individual-stack fiber");
 
 #ifdef HILTI_HAVE_ASAN
