@@ -19,6 +19,14 @@ HILTI_EXCEPTION_IMPL(Backtrack)
 HILTI_EXCEPTION_IMPL(MissingData);
 HILTI_EXCEPTION_IMPL(ParseError)
 
+void spicy::rt::Parser::_initProfiling() {
+    // Cache profiler tags to avoid recomputing them frequently.
+    assert(! name.empty());
+    profiler_tags.prepare_block = "spicy/prepare/block/" + name;
+    profiler_tags.prepare_input = "spicy/prepare/input/" + name;
+    profiler_tags.prepare_stream = "spicy/prepare/stream/" + name;
+}
+
 void spicy::rt::accept_input() {
     if ( const auto& hook = configuration::detail::unsafeGet().hook_accept_input )
         (*hook)();
