@@ -16,6 +16,7 @@
 #include <hilti/ast/expressions/resolved-operator.h>
 #include <hilti/ast/operators/function.h>
 #include <hilti/ast/operators/struct.h>
+#include <hilti/ast/types/bitfield.h>
 #include <hilti/ast/types/integer.h>
 #include <hilti/ast/types/reference.h>
 #include <hilti/ast/types/regexp.h>
@@ -161,13 +162,6 @@ struct VisitorPass2 : public hilti::visitor::PreOrder<void, VisitorPass2> {
      *     replaceNode(&p, hilti::expression::UnresolvedOperator(n.operator_().kind(), n.operands(), p.node.meta()));
      * }
      */
-    result_t operator()(const operator_::bitfield::Member& n, position_t p) {
-        const auto& id = n.op1().as<hilti::expression::Member>().id();
-        auto idx = n.op0().type().as<spicy::type::Bitfield>().bitsIndex(id);
-        assert(idx);
-        auto x = builder::index(n.op0(), *idx, n.meta());
-        replaceNode(&p, std::move(x));
-    }
 
     result_t operator()(const operator_::unit::Unset& n, position_t p) {
         const auto& id = n.op1().as<hilti::expression::Member>().id();
@@ -201,13 +195,13 @@ struct VisitorPass2 : public hilti::visitor::PreOrder<void, VisitorPass2> {
     }
 
     result_t operator()(const operator_::unit::Offset& n, position_t p) {
-        auto begin = builder::memberCall(builder::deref(builder::member(n.op0(), ID("__begin"))), "offset", {});
-        auto cur = builder::memberCall(builder::deref(builder::member(n.op0(), ID("__position"))), "offset", {});
-        replaceNode(&p, builder::grouping(builder::difference(cur, begin)));
+        replaceNode(&p, builder::member(n.op0(), ID("__offset")));
     }
 
     result_t operator()(const operator_::unit::Position& n, position_t p) {
-        replaceNode(&p, builder::member(n.op0(), ID("__position")));
+        auto begin = builder::deref(builder::member(n.op0(), ID("__begin")));
+        auto offset = builder::member(n.op0(), ID("__offset"));
+        replaceNode(&p, builder::grouping(builder::sum(begin, offset)));
     }
 
     result_t operator()(const operator_::unit::Input& n, position_t p) {
@@ -222,7 +216,8 @@ struct VisitorPass2 : public hilti::visitor::PreOrder<void, VisitorPass2> {
 
     result_t operator()(const operator_::unit::Find& n, position_t p) {
         auto begin = builder::deref(builder::member(n.op0(), ID("__begin")));
-        auto end = builder::deref(builder::member(n.op0(), ID("__position")));
+        auto offset = builder::member(n.op0(), ID("__offset"));
+        auto end = builder::sum(begin, offset);
         auto needle = argument(n.op2(), 0);
         auto direction = argument(n.op2(), 1, builder::id("spicy::Direction::Forward"));
         auto i = argument(n.op2(), 2, builder::null());
@@ -245,6 +240,12 @@ struct VisitorPass2 : public hilti::visitor::PreOrder<void, VisitorPass2> {
         replaceNode(&p, std::move(x));
     }
 
+    result_t operator()(const spicy::ctor::Unit& c, position_t p) {
+        // Replace unit ctor with an equivalent struct ctor.
+        auto n = hilti::ctor::Struct(c.fields().copy(), c.meta());
+        replaceNode(&p, n);
+    }
+
     result_t operator()(const operator_::unit::ConnectFilter& n, position_t p) {
         auto x = builder::call("spicy_rt::filter_connect", {n.op0(), argument(n.op2(), 0)});
         replaceNode(&p, std::move(x));
