@@ -22,6 +22,7 @@
 #include <spicy/ast/all.h>
 #include <spicy/ast/detail/visitor.h>
 #include <spicy/ast/hook.h>
+#include <spicy/ast/operators/unit.h>
 #include <spicy/ast/types.h>
 #include <spicy/ast/types/unit.h>
 #include <spicy/compiler/detail/visitors.h>
@@ -570,20 +571,55 @@ struct VisitorPost : public hilti::visitor::PreOrder<void, VisitorPost>, public
         }
     }
 
+    void checkBits(const spicy::type::Unit& u, position_t p, const hilti::node::Set<type::unit::Item>& items,
+                   std::set<ID>* seen_bits) {
+        for ( const auto& item : items ) {
+            if ( auto f = item.tryAs<spicy::type::unit::item::Field>() ) {
+                if ( ! f->isAnonymous() )
+                    continue;
+
+                auto t = f->itemType().tryAs<type::Bitfield>();
+                if ( ! t )
+                    continue;
+
+                for ( const auto& b : t->bits() ) {
+                    if ( u.itemByName(b.id()) )
+                        error(fmt("bitfield item '%s' shadows unit field", b.id()), p);
+
+                    if ( seen_bits->find(b.id()) != seen_bits->end() )
+                        error(fmt("bitfield item name '%s' appears in multiple anonymous bitfields", b.id()), p);
+
+                    seen_bits->insert(b.id());
+                }
+            }
+
+            else if ( auto f = item.tryAs<spicy::type::unit::item::Switch>() ) {
+                for ( const auto& c : f->cases() )
+                    checkBits(u, p, c.items(), seen_bits);
+            }
+        }
+    }
+
     void operator()(const spicy::type::Unit& u, position_t p) {
         if ( auto attrs = u.attributes() ) {
             if ( AttributeSet::find(attrs, "&size") && AttributeSet::find(attrs, "&max-size") )
                 error(("attributes cannot be combined: &size, &max-size"), p);
 
             for ( const auto& a : attrs->attributes() ) {
-                if ( a.tag() == "&size" ) {
+                if ( a.tag() == "&size" || a.tag() == "&max-size" ) {
                     if ( ! a.hasValue() )
-                        error("&size must provide an expression", p);
-                }
-
-                else if ( a.tag() == "&max-size" ) {
-                    if ( ! a.hasValue() )
-                        error("&max-size must provide an expression", p);
+                        error(fmt("%s must provide an expression", a.tag()), p);
+                    else {
+                        auto v = hilti::visitor::PreOrder<>();
+                        for ( auto i : v.walk(a.value()) )
+                            if ( const auto& id = i.node.tryAs<ID>(); id && id->str() == "self" ) {
+                                error(fmt("%s expression cannot use 'self' since it is only available after parsing of "
+                                          "unit has started",
+                                          a.tag()),
+                                      p);
+                                break;
+                            }
+                    }
                 }
 
                 else if ( a.tag() == "&requires" ) {
@@ -629,6 +665,11 @@ struct VisitorPost : public hilti::visitor::PreOrder<void, VisitorPost>, public
 
         if ( u.propertyItem("%synchronize-at") && u.propertyItem("%synchronize-after") )
             error("unit cannot specify both %synchronize-at and %synchronize-after", p);
+
+
+        // Ensure that the items of anonymous bitfields do not lead to ambiguities.
+        std::set<ID> seen_bits;
+        checkBits(u, p, u.items(), &seen_bits);
     }
 
     void operator()(const hilti::operator_::value_reference::Equal& o, position_t p) {
@@ -695,6 +736,22 @@ struct VisitorPost : public hilti::visitor::PreOrder<void, VisitorPost>, public
                 }
             }
         }
+
+        // Check for attributes which can be used at most once.
+        if ( f.attributes() ) {
+            std::unordered_map<std::string, size_t> attrs;
+            for ( const auto& a : f.attributes()->attributes() )
+                attrs[a.tag()] += 1;
+
+            for ( const auto& [a, count] : attrs ) {
+                if ( count <= 1 )
+                    continue;
+
+                if ( a == "&convert" || a == "&size" || a == "&max-size" || a == "&parse-at" || a == "&parse-from" ||
+                     a == "&type" || a == "&until" || a == "&until-including" || a == "&while" )
+                    error(fmt("'%s' can be used at most once", a), p);
+            }
+        }
     }
 
     void operator()(const spicy::type::unit::item::UnresolvedField& u, position_t p) {
@@ -911,21 +968,6 @@ struct VisitorPost : public hilti::visitor::PreOrder<void, VisitorPost>, public
         if ( auto x = n.op0().type().tryAs<type::Unit>(); x && ! x->isFilter() )
             error("unit type cannot be a filter, %filter missing", p);
     }
-
-    void operator()(const spicy::type::Bitfield& b, position_t p) {
-        const auto width = b.width();
-
-        for ( const auto& bit : b.bits() ) {
-            const auto lower = bit.lower();
-            const auto upper = bit.upper();
-
-            if ( lower > upper )
-                error("lower limit needs to be lower than upper limit", p);
-
-            if ( upper >= width )
-                error("upper limit is beyond the width of the bitfield", p);
-        }
-    }
 };
 
 } // anonymous namespace
