@@ -205,7 +205,7 @@ void get_first_statement(Stmt* list, Stmt*& first, Location& loc)
 	while ( first->Tag() == STMT_LIST )
 		{
 		if ( first->AsStmtList()->Stmts()[0] )
-			first = first->AsStmtList()->Stmts()[0];
+			first = first->AsStmtList()->Stmts()[0].get();
 		else
 			break;
 		}
@@ -342,7 +342,7 @@ static void parse_function_name(vector<ParseLocationRec>& result, ParseLocationR
 vector<ParseLocationRec> parse_location_string(const string& s)
 	{
 	vector<ParseLocationRec> result;
-	result.push_back(ParseLocationRec());
+	result.emplace_back();
 	ParseLocationRec& plr = result[0];
 
 	// If PLR_FILE_AND_LINE, set this to the filename you want; for
@@ -485,7 +485,7 @@ int dbg_shutdown_debugger()
 // by the operation argument; the additional arguments are put in the
 // supplied vector.
 //
-// Parse the string into individual tokens, similarily to how shell
+// Parse the string into individual tokens, similarly to how shell
 // would do it.
 
 void tokenize(const char* cstr, string& operation, vector<string>& arguments)
@@ -531,7 +531,7 @@ void tokenize(const char* cstr, string& operation, vector<string>& arguments)
 		if ( ! num_tokens )
 			operation = string(str, start, len);
 		else
-			arguments.push_back(string(str, start, len));
+			arguments.emplace_back(str, start, len);
 
 		++num_tokens;
 		}
