@@ -427,7 +427,9 @@ public:
 
 	RobustDictIterator& operator++()
 		{
-		curr = dict->GetNextRobustIteration(this);
+		if ( dict )
+			curr = dict->GetNextRobustIteration(this);
+
 		return *this;
 		}
 
@@ -531,6 +533,7 @@ private:
 			inserted = nullptr;
 			visited = nullptr;
 			dict = nullptr;
+			curr = nullptr; // make this same as robust_end()
 			}
 		}
 
@@ -855,6 +858,11 @@ public:
 
 		if ( iterators )
 			{
+			// Complete() erases from this Dictionary's iterators member, use a copy.
+			auto copied_iterators = *iterators;
+			for ( auto* i : copied_iterators )
+				i->Complete();
+
 			delete iterators;
 			iterators = nullptr;
 			}
