@@ -53,6 +53,13 @@ public:
 	 */
 	Frame(int size, const ScriptFunc* func, const zeek::Args* fn_args);
 
+	/**
+	 * Returns the size of the frame.
+	 *
+	 * @return the number of elements in the frame.
+	 */
+	int FrameSize() const { return size; }
+
 	/**
 	 * @param n the index to get.
 	 * @return the value at index *n* of the underlying array.
@@ -165,16 +172,11 @@ public:
 	Frame* CloneForTrigger() const;
 
 	/**
-	 * Serializes the frame in support of copy semantics for lambdas:
-	 *
-	 * [ "CopyFrame", serialized_values ]
-	 *
-	 * where serialized_values are two-element vectors. A serialized_value
-	 * has the result of calling broker::data_to_val on the value in the
-	 * first index, and an integer representing that value's type in the
-	 * second index.
+	 * Serializes the frame (only done for lambda/when captures) as a
+	 * sequence of two-element vectors, the first element reflecting
+	 * the frame value, the second its type.
 	 */
-	broker::expected<broker::data> SerializeCopyFrame();
+	broker::expected<broker::data> Serialize();
 
 	/**
 	 * Instantiates a Frame from a serialized one.
@@ -182,13 +184,8 @@ public:
 	 * @return a pair in which the first item is the status of the serialization;
 	 * and the second is the unserialized frame with reference count +1, or
 	 * null if the serialization wasn't successful.
-	 *
-	 * The *captures* argument, if non-nil, specifies that the frame
-	 * reflects captures with copy-semantics rather than deprecated
-	 * reference semantics.
 	 */
-	static std::pair<bool, FramePtr>
-	Unserialize(const broker::vector& data, const std::optional<FuncType::CaptureList>& captures);
+	static std::pair<bool, FramePtr> Unserialize(const broker::vector& data);
 
 	// If the frame is run in the context of a trigger condition evaluation,
 	// the trigger needs to be registered.
@@ -239,7 +236,7 @@ private:
 	 */
 	int current_offset;
 
-	/** Frame used for captures (if any) with copy semantics. */
+	/** Frame used for lambda/when captures. */
 	Frame* captures;
 
 	/** Maps IDs to offsets into the "captures" frame.  If the ID
