@@ -132,8 +132,8 @@ ID::ID(const char* arg_name, IDScope arg_scope, bool arg_is_export)
 
 ID::~ID()
 	{
+	ClearOptInfo();
 	delete[] name;
-	delete opt_info;
 	}
 
 std::string ID::ModuleName() const
@@ -687,6 +687,12 @@ std::vector<Func*> ID::GetOptionHandlers() const
 	return v;
 	}
 
+void ID::ClearOptInfo()
+	{
+	delete opt_info;
+	opt_info = nullptr;
+	}
+
 	} // namespace detail
 
 	} // namespace zeek
