@@ -5,6 +5,7 @@
 #include <map>
 #include <string>
 #include <string_view>
+#include <unordered_set>
 #include <vector>
 
 #include "zeek/Attr.h"
@@ -45,6 +46,7 @@ enum InitClass
 	INIT_FULL,
 	INIT_EXTRA,
 	INIT_REMOVE,
+	INIT_SKIP,
 	};
 enum IDScope
 	{
@@ -55,6 +57,7 @@ enum IDScope
 
 class ID;
 using IDPtr = IntrusivePtr<ID>;
+using IDSet = std::unordered_set<const ID*>;
 
 class IDOptInfo;
 
@@ -103,6 +106,7 @@ public:
 
 	void SetOption();
 	bool IsOption() const { return is_option; }
+	bool IsBlank() const { return is_blank; };
 
 	void SetEnumConst() { is_enum_const = true; }
 	bool IsEnumConst() const { return is_enum_const; }
@@ -113,6 +117,7 @@ public:
 	bool IsRedefinable() const;
 
 	void SetAttrs(AttributesPtr attr);
+	void AddAttr(AttrPtr a, bool is_redef = false);
 	void AddAttrs(AttributesPtr attr, bool is_redef = false);
 	void RemoveAttr(AttrTag a);
 	void UpdateValAttrs();
@@ -139,7 +144,7 @@ public:
 	bool DoInferReturnType() const { return infer_return_type; }
 	void SetInferReturnType(bool infer) { infer_return_type = infer; }
 
-	virtual TraversalCode Traverse(TraversalCallback* cb) const;
+	TraversalCode Traverse(TraversalCallback* cb) const;
 
 	bool HasOptionHandlers() const { return ! option_handlers.empty(); }
 
@@ -147,6 +152,7 @@ public:
 	std::vector<Func*> GetOptionHandlers() const;
 
 	IDOptInfo* GetOptInfo() const { return opt_info; }
+	void ClearOptInfo();
 
 protected:
 	void EvalFunc(ExprPtr ef, ExprPtr ev);
@@ -160,7 +166,7 @@ protected:
 	bool is_export;
 	bool infer_return_type;
 	TypePtr type;
-	bool is_const, is_enum_const, is_type, is_option;
+	bool is_const, is_enum_const, is_type, is_option, is_blank;
 	int offset;
 	ValPtr val;
 	AttributesPtr attrs;
