@@ -11,6 +11,7 @@
 #include "zeek/Desc.h"
 #include "zeek/Reporter.h"
 #include "zeek/Type.h"
+#include "zeek/script_opt/ScriptOpt.h"
 
 using namespace std;
 
@@ -21,15 +22,18 @@ ScriptCoverageManager::ScriptCoverageManager() : ignoring(0), delim('\t') { }
 
 void ScriptCoverageManager::AddStmt(Stmt* s)
 	{
-	if ( ignoring != 0 )
+	if ( ignoring != 0 || analysis_options.gen_ZAM )
 		return;
 
-	stmts.push_back({NewRef{}, s});
+	stmts.emplace_back(NewRef{}, s);
 	}
 
 void ScriptCoverageManager::AddFunction(IDPtr func_id, StmtPtr body)
 	{
-	func_instances.push_back({func_id, body});
+	if ( analysis_options.gen_ZAM )
+		return;
+
+	func_instances.emplace_back(func_id, body);
 	}
 
 bool ScriptCoverageManager::ReadStats()
