@@ -4,8 +4,10 @@
 
 #include "zeek/List.h"
 
-namespace zeek {
-namespace detail {
+namespace zeek
+	{
+namespace detail
+	{
 
 class Expr;
 class ID;
@@ -13,7 +15,7 @@ class Stmt;
 class Attr;
 class Timer;
 
-} // namespace detail
+	} // namespace detail
 
 class Val;
 class Type;
@@ -21,9 +23,8 @@ class Type;
 using ValPList = PList<Val>;
 using ExprPList = PList<detail::Expr>;
 using IDPList = PList<detail::ID>;
-using StmtPList = PList<detail::Stmt>;
 using TypePList = PList<Type>;
 using AttrPList = PList<detail::Attr>;
 using TimerPList = PList<detail::Timer, ListOrder::UNORDERED>;
 
-} // namespace zeek
+	} // namespace zeek
