@@ -3,6 +3,8 @@
 
 #include "zeek/analyzer/protocol/pop3/POP3.h"
 
+#include "zeek/zeek-config.h"
+
 #include <cctype>
 #include <string>
 #include <vector>
@@ -10,7 +12,6 @@
 #include "zeek/Base64.h"
 #include "zeek/Reporter.h"
 #include "zeek/analyzer/Manager.h"
-#include "zeek/analyzer/protocol/pop3/consts.bif.h"
 #include "zeek/analyzer/protocol/pop3/events.bif.h"
 
 namespace zeek::analyzer::pop3
@@ -43,7 +44,6 @@ POP3_Analyzer::POP3_Analyzer(Connection* conn)
 	authLines = 0;
 
 	mail = nullptr;
-	unknown_client_cmds = 0;
 
 	cl_orig = new analyzer::tcp::ContentLine_Analyzer(conn, true);
 	AddSupportAnalyzer(cl_orig);
@@ -52,8 +52,6 @@ POP3_Analyzer::POP3_Analyzer(Connection* conn)
 	AddSupportAnalyzer(cl_resp);
 	}
 
-POP3_Analyzer::~POP3_Analyzer() { }
-
 void POP3_Analyzer::Done()
 	{
 	analyzer::tcp::TCP_ApplicationAnalyzer::Done();
@@ -224,22 +222,7 @@ void POP3_Analyzer::ProcessRequest(int length, const char* line)
 		// Some clients pipeline their commands (i.e., keep sending
 		// without waiting for a server's responses). Therefore we
 		// keep a list of pending commands.
-		cmds.push_back(std::string(line));
-
-		// Prevent unbounded state growth of cmds if there are no matching
-		// server replies by simply dropping the oldest command.
-		//
-		// This may be caused by packet drops of the server side, one-sided
-		// traffic, or analyzing the wrong protocol (Redis), etc.
-		if ( zeek::BifConst::POP3::max_pending_commands > 0 )
-			{
-			if ( cmds.size() > zeek::BifConst::POP3::max_pending_commands )
-				{
-				Weird("pop3_client_too_many_pending_commands");
-
-				cmds.pop_front();
-				}
-			}
+		cmds.emplace_back(line);
 
 		if ( cmds.size() == 1 )
 			// Not waiting for another server response,
@@ -276,20 +259,10 @@ void POP3_Analyzer::ProcessClientCmd()
 		{
 		if ( ! waitingForAuthentication )
 			{
-			Weird("pop3_client_command_unknown", (tokens.size() > 0 ? tokens[0].c_str() : "???"));
+			Weird("pop3_client_command_unknown");
 			if ( subState == detail::POP3_WOK )
 				subState = detail::POP3_OK;
-
-			++unknown_client_cmds;
-			if ( zeek::BifConst::POP3::max_unknown_client_commands > 0 )
-				{
-				if ( unknown_client_cmds > zeek::BifConst::POP3::max_unknown_client_commands )
-					{
-					AnalyzerViolation("too many unknown client commands");
-					}
-				}
 			}
-
 		return;
 		}
 
@@ -358,7 +331,6 @@ void POP3_Analyzer::ProcessClientCmd()
 				POP3Event(pop3_request, true, cmd, message);
 				if ( ! *message )
 					{
-					// This is the client requesting a list of AUTH mechanisms available.
 					requestForMultiLine = true;
 					state = detail::AUTH;
 					subState = detail::POP3_WOK;
@@ -655,11 +627,10 @@ void POP3_Analyzer::ProcessReply(int length, const char* line)
 			                            (tokens.size() > 0 ? tokens[0].c_str() : "???")),
 			                  line, length);
 
-			Weird("pop3_server_command_unknown", (tokens.size() > 0 ? tokens[0].c_str() : "???"));
+			Weird("pop3_server_command_unknown");
 			if ( subState == detail::POP3_WOK )
 				subState = detail::POP3_OK;
 			}
-
 		return;
 		}
 
@@ -833,7 +804,7 @@ void POP3_Analyzer::ProcessReply(int length, const char* line)
 
 void POP3_Analyzer::StartTLS()
 	{
-	// STARTTLS was succesful. Remove support analyzers, add SSL
+	// STARTTLS was successful. Remove support analyzers, add SSL
 	// analyzer, and throw event signifying the change.
 	RemoveSupportAnalyzer(cl_orig);
 	RemoveSupportAnalyzer(cl_resp);
