@@ -17,8 +17,6 @@ Component::Component(plugin::component::Type type, const std::string& name)
 	{
 	}
 
-Component::~Component() { }
-
 PktSrcComponent::PktSrcComponent(const std::string& arg_name, const std::string& arg_prefix,
                                  InputType arg_type, factory_callback arg_factory)
 	: Component(plugin::component::PKTSRC, arg_name)
@@ -28,8 +26,6 @@ PktSrcComponent::PktSrcComponent(const std::string& arg_name, const std::string&
 	factory = arg_factory;
 	}
 
-PktSrcComponent::~PktSrcComponent() { }
-
 const std::vector<std::string>& PktSrcComponent::Prefixes() const
 	{
 	return prefixes;
@@ -110,8 +106,6 @@ PktDumperComponent::PktDumperComponent(const std::string& name, const std::strin
 	factory = arg_factory;
 	}
 
-PktDumperComponent::~PktDumperComponent() { }
-
 PktDumperComponent::factory_callback PktDumperComponent::Factory() const
 	{
 	return factory;
