@@ -27,7 +27,7 @@ public:
 	 *
 	 * @param factory A factory function to instantiate instances of the
 	 * writers's class, which must be derived directly or indirectly from
-	 * logging::WriterBackend. This is typically a static \c Instatiate()
+	 * logging::WriterBackend. This is typically a static \c Instantiate()
 	 * method inside the class that just allocates and returns a new
 	 * instance.
 	 */
@@ -36,7 +36,7 @@ public:
 	/**
 	 * Destructor.
 	 */
-	~Component() override;
+	~Component() override = default;
 
 	/**
 	 * Initialization function. This function has to be called before any
@@ -52,7 +52,7 @@ public:
 
 protected:
 	/**
-	 * Overriden from plugin::Component.
+	 * Overridden from plugin::Component.
 	 */
 	void DoDescribe(ODesc* d) const override;
 
