@@ -1,17 +1,20 @@
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #include "zeek/plugin/Plugin.h"
+
 #include "zeek/packet_analysis/Component.h"
 #include "zeek/packet_analysis/protocol/skip/Skip.h"
 
-namespace zeek::plugin::Zeek_Skip {
+namespace zeek::plugin::Zeek_Skip
+	{
 
-class Plugin : public zeek::plugin::Plugin {
+class Plugin final : public zeek::plugin::Plugin
+	{
 public:
-	zeek::plugin::Configuration Configure()
+	zeek::plugin::Configuration Configure() override
 		{
-		AddComponent(new zeek::packet_analysis::Component("Skip",
-		                 zeek::packet_analysis::Skip::SkipAnalyzer::Instantiate));
+		AddComponent(new zeek::packet_analysis::Component(
+			"Skip", zeek::packet_analysis::Skip::SkipAnalyzer::Instantiate));
 
 		zeek::plugin::Configuration config;
 		config.name = "Zeek::Skip";
@@ -19,6 +22,6 @@ public:
 		return config;
 		}
 
-} plugin;
+	} plugin;
 
-}
+	}
