@@ -10,10 +10,10 @@
 namespace zeek::plugin::Zeek_TCP
 	{
 
-class Plugin : public zeek::plugin::Plugin
+class Plugin final : public zeek::plugin::Plugin
 	{
 public:
-	zeek::plugin::Configuration Configure()
+	zeek::plugin::Configuration Configure() override
 		{
 		AddComponent(new zeek::packet_analysis::Component(
 			"TCP", zeek::packet_analysis::TCP::TCPAnalyzer::Instantiate));
