@@ -27,7 +27,6 @@ const char* hook_name(HookType h)
 		"QueueEvent",
 		"DrainEvents",
 		"UpdateNetworkTime",
-		"BroObjDtor",
 		"SetupAnalyzerTree",
 		"LogInit",
 		"LogWrite",
@@ -67,8 +66,6 @@ BifItem& BifItem::operator=(const BifItem& other)
 	return *this;
 	}
 
-BifItem::~BifItem() { }
-
 void HookArgument::Describe(ODesc* d) const
 	{
 	switch ( type )
@@ -381,18 +378,6 @@ void Plugin::RequestEvent(EventHandlerPtr handler)
 	plugin_mgr->RequestEvent(handler, this);
 	}
 
-void Plugin::RequestBroObjDtor(Obj* obj)
-	{
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-#endif
-	plugin_mgr->RequestBroObjDtor(obj, this);
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
-	}
-
 void Plugin::RequestObjDtor(Obj* obj)
 	{
 	plugin_mgr->RequestObjDtor(obj, this);
@@ -427,8 +412,6 @@ void Plugin::HookUpdateNetworkTime(double network_time) { }
 
 void Plugin::HookSetupAnalyzerTree(Connection* conn) { }
 
-void Plugin::HookBroObjDtor(void* obj) { }
-
 void Plugin::HookObjDtor(void* obj) { }
 
 void Plugin::HookLogInit(const std::string& writer, const std::string& instantiating_filter,
