@@ -27,12 +27,15 @@ public:
 		Analyze();
 		}
 
-	// Either returns the original CallExpr if it's not inline-able,
-	// or an InlineExpr if it is.
+	// Either returns the original CallExpr if it's not inline-able;
+	// or an InlineExpr if it is; or nil if further inlining should stop.
 	ExprPtr CheckForInlining(CallExprPtr c);
 
-	// True if the given function has been inlined.
-	bool WasInlined(const Func* f) { return inline_ables.count(f) > 0; }
+	// True if every instance of the function was inlined.
+	bool WasFullyInlined(const Func* f)
+		{
+		return did_inline.count(f) > 0 && skipped_inlining.count(f) == 0;
+		}
 
 protected:
 	// Driver routine that analyzes all of the script functions and
@@ -49,6 +52,13 @@ protected:
 	// Functions that we've determined to be suitable for inlining.
 	std::unordered_set<const Func*> inline_ables;
 
+	// Functions that we inlined.
+	std::unordered_set<const Func*> did_inline;
+
+	// Functions that we didn't fully inline, so require separate
+	// compilation.
+	std::unordered_set<const Func*> skipped_inlining;
+
 	// As we do inlining for a given function, this tracks the
 	// largest frame size of any inlined function.
 	int max_inlined_frame_size;
@@ -57,6 +67,12 @@ protected:
 	// prior to increasing it to accommodate inlining.
 	int curr_frame_size;
 
+	// The number of statements and expressions in the function being
+	// inlined.  Dynamically updated as the inlining proceeds.  Used
+	// to cap inlining complexity.
+	int num_stmts;
+	int num_exprs;
+
 	// Whether to generate a report about functions either directly and
 	// indirectly recursive.
 	bool report_recursive;
