@@ -91,7 +91,7 @@ StmtPtr ExprListStmt::DoReduce(Reducer* c)
 			new_l->Append(red_e);
 
 			if ( red_e_stmt )
-				s->Stmts().push_back(red_e_stmt.release());
+				s->Stmts().push_back(red_e_stmt);
 			}
 		}
 
@@ -103,7 +103,7 @@ StmtPtr ExprListStmt::DoReduce(Reducer* c)
 
 	else
 		{
-		s->Stmts().push_back(DoSubclassReduce(new_l, c).release());
+		s->Stmts().push_back(DoSubclassReduce(new_l, c));
 		return s->Reduce(c);
 		}
 	}
@@ -296,7 +296,7 @@ StmtPtr IfStmt::DoReduce(Reducer* c)
 		}
 
 	if ( red_e_stmt )
-		return TransformMe(make_intrusive<StmtList>(red_e_stmt, this), c);
+		return TransformMe(make_intrusive<StmtList>(red_e_stmt, ThisPtr()), c);
 
 	return ThisPtr();
 	}
@@ -320,13 +320,20 @@ IntrusivePtr<Case> Case::Duplicate()
 		return make_intrusive<Case>(new_exprs, nullptr, s->Duplicate());
 		}
 
+	IDPList* new_type_cases = nullptr;
+
 	if ( type_cases )
 		{
+		new_type_cases = new IDPList();
+
 		for ( auto tc : *type_cases )
+			{
 			zeek::Ref(tc);
+			new_type_cases->append(tc);
+			}
 		}
 
-	return make_intrusive<Case>(nullptr, type_cases, s->Duplicate());
+	return make_intrusive<Case>(nullptr, new_type_cases, s->Duplicate());
 	}
 
 StmtPtr SwitchStmt::Duplicate()
@@ -354,6 +361,9 @@ bool SwitchStmt::IsReduced(Reducer* r) const
 	if ( ! e->IsReduced(r) )
 		return NonReduced(e.get());
 
+	if ( cases->length() == 0 )
+		return false;
+
 	for ( const auto& c : *cases )
 		{
 		if ( c->ExprCases() && ! c->ExprCases()->IsReduced(r) )
@@ -371,6 +381,10 @@ bool SwitchStmt::IsReduced(Reducer* r) const
 
 StmtPtr SwitchStmt::DoReduce(Reducer* rc)
 	{
+	if ( cases->length() == 0 )
+		// Degenerate.
+		return make_intrusive<NullStmt>();
+
 	auto s = make_intrusive<StmtList>();
 	StmtPtr red_e_stmt;
 
@@ -382,7 +396,15 @@ StmtPtr SwitchStmt::DoReduce(Reducer* rc)
 	// Note, the compiler checks for constant switch expressions.
 
 	if ( red_e_stmt )
-		s->Stmts().push_back(red_e_stmt.release());
+		s->Stmts().push_back(red_e_stmt);
+
+	// Update type cases.
+	for ( auto& i : case_label_type_list )
+		{
+		IDPtr idp = {NewRef{}, i.first};
+		if ( idp->Name() )
+			i.first = rc->UpdateID(idp).release();
+		}
 
 	for ( const auto& c : *cases )
 		{
@@ -393,24 +415,21 @@ StmtPtr SwitchStmt::DoReduce(Reducer* rc)
 			auto red_cases = c_e->Reduce(rc, c_e_stmt);
 
 			if ( c_e_stmt )
-				s->Stmts().push_back(c_e_stmt.release());
+				s->Stmts().push_back(c_e_stmt);
 			}
 
 		auto c_t = c->TypeCases();
 		if ( c_t )
-			rc->UpdateIDs(c_t);
+			{
+			for ( auto& c_t_i : *c_t )
+				if ( c_t_i->Name() )
+					c_t_i = rc->UpdateID({NewRef{}, c_t_i}).release();
+			}
 
 		c->UpdateBody(c->Body()->Reduce(rc));
 		}
 
-	// Update type cases.
-	for ( auto& i : case_label_type_list )
-		{
-		IDPtr idp = {NewRef{}, i.first};
-		i.first = rc->UpdateID(idp).release();
-		}
-
-	if ( s->Stmts().length() > 0 )
+	if ( ! s->Stmts().empty() )
 		{
 		StmtPtr me = ThisPtr();
 		auto pre_and_me = make_intrusive<StmtList>(s, me);
@@ -634,7 +653,7 @@ StmtPtr ForStmt::DoReduce(Reducer* c)
 		Warn("empty \"for\" body leaves loop variables in indeterminate state");
 
 	if ( red_e_stmt )
-		return TransformMe(make_intrusive<StmtList>(red_e_stmt, this), c);
+		return TransformMe(make_intrusive<StmtList>(red_e_stmt, ThisPtr()), c);
 
 	return ThisPtr();
 	}
@@ -646,21 +665,26 @@ StmtPtr ReturnStmt::Duplicate()
 
 ReturnStmt::ReturnStmt(ExprPtr arg_e, bool ignored) : ExprStmt(STMT_RETURN, std::move(arg_e)) { }
 
+bool ReturnStmt::IsReduced(Reducer* c) const
+	{
+	if ( ! e || e->IsSingleton(c) )
+		return true;
+
+	return NonReduced(e.get());
+	}
+
 StmtPtr ReturnStmt::DoReduce(Reducer* c)
 	{
 	if ( ! e )
 		return ThisPtr();
 
 	if ( c->Optimizing() )
-		{
 		e = c->OptExpr(e);
-		return ThisPtr();
-		}
 
-	if ( ! e->IsSingleton(c) )
+	else if ( ! e->IsSingleton(c) )
 		{
 		StmtPtr red_e_stmt;
-		e = e->Reduce(c, red_e_stmt);
+		e = e->ReduceToSingleton(c, red_e_stmt);
 
 		if ( red_e_stmt )
 			{
@@ -672,58 +696,47 @@ StmtPtr ReturnStmt::DoReduce(Reducer* c)
 	return ThisPtr();
 	}
 
-StmtList::StmtList(StmtPtr s1, Stmt* s2) : Stmt(STMT_LIST)
-	{
-	stmts = new StmtPList;
-	if ( s1 )
-		stmts->append(s1.release());
-	if ( s2 )
-		stmts->append(s2);
-	}
-
 StmtList::StmtList(StmtPtr s1, StmtPtr s2) : Stmt(STMT_LIST)
 	{
-	stmts = new StmtPList;
 	if ( s1 )
-		stmts->append(s1.release());
+		stmts.push_back(std::move(s1));
 	if ( s2 )
-		stmts->append(s2.release());
+		stmts.push_back(std::move(s2));
 	}
 
 StmtList::StmtList(StmtPtr s1, StmtPtr s2, StmtPtr s3) : Stmt(STMT_LIST)
 	{
-	stmts = new StmtPList;
 	if ( s1 )
-		stmts->append(s1.release());
+		stmts.push_back(std::move(s1));
 	if ( s2 )
-		stmts->append(s2.release());
+		stmts.push_back(std::move(s2));
 	if ( s3 )
-		stmts->append(s3.release());
+		stmts.push_back(std::move(s3));
 	}
 
 StmtPtr StmtList::Duplicate()
 	{
 	auto new_sl = new StmtList();
 
-	for ( auto& stmt : Stmts() )
-		new_sl->Stmts().push_back(stmt->Duplicate().release());
+	for ( auto& stmt : stmts )
+		new_sl->stmts.push_back(stmt->Duplicate());
 
 	return SetSucc(new_sl);
 	}
 
 void StmtList::Inline(Inliner* inl)
 	{
-	for ( const auto& stmt : Stmts() )
+	for ( const auto& stmt : stmts )
 		stmt->Inline(inl);
 	}
 
 bool StmtList::IsReduced(Reducer* c) const
 	{
-	int n = Stmts().length();
+	auto n = stmts.size();
 
 	for ( auto i = 0; i < n; ++i )
 		{
-		auto& s_i = Stmts()[i];
+		auto& s_i = stmts[i];
 		if ( ! s_i->IsReduced(c) )
 			return false;
 
@@ -736,17 +749,17 @@ bool StmtList::IsReduced(Reducer* c) const
 
 StmtPtr StmtList::DoReduce(Reducer* c)
 	{
-	StmtPList* f_stmts = new StmtPList{};
+	std::vector<StmtPtr> f_stmts;
 	bool did_change = false;
 
-	int n = Stmts().length();
+	auto n = stmts.size();
 
 	for ( auto i = 0; i < n; ++i )
 		{
 		if ( ReduceStmt(i, f_stmts, c) )
 			did_change = true;
 
-		if ( i < n - 1 && Stmts()[i]->NoFlowAfter(false) )
+		if ( i < n - 1 && stmts[i]->NoFlowAfter(false) )
 			{
 			did_change = true;
 			break;
@@ -756,34 +769,28 @@ StmtPtr StmtList::DoReduce(Reducer* c)
 			return ThisPtr();
 		}
 
-	if ( f_stmts->length() == 0 )
-		{
-		delete f_stmts;
+	if ( f_stmts.empty() )
 		return TransformMe(make_intrusive<NullStmt>(), c);
-		}
 
-	if ( f_stmts->length() == 1 )
-		return (*f_stmts)[0]->Reduce(c);
+	if ( f_stmts.size() == 1 )
+		return f_stmts[0]->Reduce(c);
 
 	if ( did_change )
 		{
-		ResetStmts(f_stmts);
+		ResetStmts(std::move(f_stmts));
 		return Reduce(c);
 		}
-	else
-		delete f_stmts;
 
 	return ThisPtr();
 	}
 
-bool StmtList::ReduceStmt(int& s_i, StmtPList* f_stmts, Reducer* c)
+bool StmtList::ReduceStmt(int& s_i, std::vector<StmtPtr>& f_stmts, Reducer* c)
 	{
 	bool did_change = false;
-	auto stmt = Stmts()[s_i]->ThisPtr();
+	auto& stmt_i = stmts[s_i];
+	auto old_stmt = stmt_i.get();
 
-	auto old_stmt = stmt;
-
-	stmt = stmt->Reduce(c);
+	auto stmt = stmt_i->Reduce(c);
 
 	if ( stmt != old_stmt )
 		did_change = true;
@@ -809,7 +816,7 @@ bool StmtList::ReduceStmt(int& s_i, StmtPList* f_stmts, Reducer* c)
 
 		if ( e->Tag() != EXPR_ASSIGN )
 			{
-			f_stmts->append(stmt.release());
+			f_stmts.push_back(std::move(stmt));
 			return false;
 			}
 
@@ -818,17 +825,17 @@ bool StmtList::ReduceStmt(int& s_i, StmtPList* f_stmts, Reducer* c)
 
 		if ( lhs->Tag() != EXPR_NAME )
 			{
-			f_stmts->append(stmt.release());
+			f_stmts.push_back(std::move(stmt));
 			return false;
 			}
 
 		auto var = lhs->AsNameExpr();
 		auto rhs = a->GetOp2();
 
-		if ( s_i < Stmts().length() - 1 )
+		if ( s_i < stmts.size() - 1 )
 			{
 			// See if we can compress an assignment chain.
-			auto& s_i_succ = Stmts()[s_i + 1];
+			auto& s_i_succ = stmts[s_i + 1];
 
 			// Don't reduce s_i_succ.  If it's what we're
 			// looking for, it's already reduced.  Plus
@@ -837,7 +844,7 @@ bool StmtList::ReduceStmt(int& s_i, StmtPList* f_stmts, Reducer* c)
 			auto merge = c->MergeStmts(var, rhs, s_i_succ);
 			if ( merge )
 				{
-				f_stmts->append(merge.release());
+				f_stmts.push_back(std::move(merge));
 
 				// Skip both this statement and the next,
 				// now that we've substituted the merge.
@@ -859,7 +866,7 @@ bool StmtList::ReduceStmt(int& s_i, StmtPList* f_stmts, Reducer* c)
 		auto sl = stmt->AsStmtList();
 
 		for ( auto& sub_stmt : sl->Stmts() )
-			f_stmts->append(sub_stmt->Ref());
+			f_stmts.push_back(sub_stmt);
 
 		did_change = true;
 		}
@@ -869,17 +876,14 @@ bool StmtList::ReduceStmt(int& s_i, StmtPList* f_stmts, Reducer* c)
 		did_change = true;
 
 	else
-		// No need to Ref() because the StmtPList destructor
-		// doesn't Unref(), only the explicit list-walking
-		// in the ~StmtList destructor.
-		f_stmts->append(stmt.release());
+		f_stmts.push_back(std::move(stmt));
 
 	return did_change;
 	}
 
 bool StmtList::NoFlowAfter(bool ignore_break) const
 	{
-	for ( auto& s : Stmts() )
+	for ( auto& s : stmts )
 		{
 		// For "break" statements, if ignore_break is set then
 		// by construction flow *does* go to after this statement
@@ -919,34 +923,100 @@ StmtPtr InitStmt::DoReduce(Reducer* c)
 	return ThisPtr();
 	}
 
-StmtPtr WhenStmt::Duplicate()
+StmtPtr AssertStmt::Duplicate()
 	{
-	FuncType::CaptureList* cl_dup = nullptr;
-
-	if ( wi->Captures() )
-		{
-		cl_dup = new FuncType::CaptureList;
-		*cl_dup = *wi->Captures();
-		}
-
-	auto new_wi = new WhenInfo(Cond(), cl_dup, IsReturn());
-	new_wi->AddBody(Body());
-	new_wi->AddTimeout(TimeoutExpr(), TimeoutBody());
-
-	return SetSucc(new WhenStmt(wi));
+	return SetSucc(new AssertStmt(cond->Duplicate(), msg ? msg->Duplicate() : nullptr));
 	}
 
-void WhenStmt::Inline(Inliner* inl)
+bool AssertStmt::IsReduced(Reducer* c) const
 	{
-	// Don't inline, since we currently don't correctly capture
-	// the frames of closures.
+	return false;
+	}
+
+StmtPtr AssertStmt::DoReduce(Reducer* c)
+	{
+	return make_intrusive<NullStmt>();
+	}
+
+bool WhenInfo::HasUnreducedIDs(Reducer* c) const
+	{
+	for ( auto& cp : *cl )
+		{
+		auto cid = cp.Id();
+
+		if ( when_new_locals.count(cid.get()) == 0 && ! c->ID_IsReduced(cp.Id()) )
+			return true;
+		}
+
+	for ( auto& l : when_expr_locals )
+		if ( ! c->ID_IsReduced(l) )
+			return true;
+
+	return false;
+	}
+
+void WhenInfo::UpdateIDs(Reducer* c)
+	{
+	for ( auto& cp : *cl )
+		{
+		auto& cid = cp.Id();
+		if ( when_new_locals.count(cid.get()) == 0 )
+			cp.SetID(c->UpdateID(cid));
+		}
+
+	for ( auto& l : when_expr_locals )
+		l = c->UpdateID(l);
+	}
+
+StmtPtr WhenStmt::Duplicate()
+	{
+	return SetSucc(new WhenStmt(std::make_shared<WhenInfo>(wi.get())));
 	}
 
 bool WhenStmt::IsReduced(Reducer* c) const
 	{
-	// We consider these always reduced because they're not
-	// candidates for any further optimization.
-	return true;
+	if ( wi->HasUnreducedIDs(c) )
+		return false;
+
+	if ( ! wi->Lambda()->IsReduced(c) )
+		return false;
+
+	if ( ! wi->TimeoutExpr() )
+		return true;
+
+	return wi->TimeoutExpr()->IsReduced(c);
+	}
+
+StmtPtr WhenStmt::DoReduce(Reducer* c)
+	{
+	if ( ! c->Optimizing() )
+		{
+		wi->UpdateIDs(c);
+		(void)wi->Lambda()->ReduceToSingletons(c);
+		}
+
+	auto e = wi->TimeoutExpr();
+
+	if ( ! e )
+		return ThisPtr();
+
+	if ( c->Optimizing() )
+		wi->SetTimeoutExpr(c->OptExpr(e));
+
+	else if ( ! e->IsSingleton(c) )
+		{
+		StmtPtr red_e_stmt;
+		auto new_e = e->ReduceToSingleton(c, red_e_stmt);
+		wi->SetTimeoutExpr(new_e);
+
+		if ( red_e_stmt )
+			{
+			auto s = make_intrusive<StmtList>(red_e_stmt, ThisPtr());
+			return TransformMe(std::move(s), c);
+			}
+		}
+
+	return ThisPtr();
 	}
 
 CatchReturnStmt::CatchReturnStmt(StmtPtr _block, NameExprPtr _ret_var) : Stmt(STMT_CATCH_RETURN)
