@@ -29,24 +29,21 @@ ZAMCompiler::ZAMCompiler(ScriptFunc* f, std::shared_ptr<ProfileFunc> _pf, ScopeP
 	Init();
 	}
 
+ZAMCompiler::~ZAMCompiler()
+	{
+	curr_stmt = nullptr;
+
+	for ( auto i : insts1 )
+		delete i;
+	}
+
 void ZAMCompiler::Init()
 	{
 	InitGlobals();
 	InitArgs();
+	InitCaptures();
 	InitLocals();
 
-#if 0
-	// Complain about unused aggregates ... but not if we're inlining,
-	// as that can lead to optimizations where they wind up being unused
-	// but the original logic for using them was sound.
-	if ( ! analysis_options.inliner )
-		for ( auto a : pf->Inits() )
-			{
-			if ( pf->Locals().find(a) == pf->Locals().end() )
-				reporter->Warning("%s unused", a->Name());
-			}
-#endif
-
 	TrackMemoryManagement();
 
 	non_recursive = non_recursive_funcs.count(func) > 0;
@@ -92,12 +89,25 @@ void ZAMCompiler::InitArgs()
 	pop_scope();
 	}
 
+void ZAMCompiler::InitCaptures()
+	{
+	for ( auto c : pf->Captures() )
+		(void)AddToFrame(c);
+	}
+
 void ZAMCompiler::InitLocals()
 	{
 	// Assign slots for locals (which includes temporaries).
 	for ( auto l : pf->Locals() )
 		{
+		if ( IsCapture(l) )
+			continue;
+
+		if ( pf->WhenLocals().count(l) > 0 )
+			continue;
+
 		auto non_const_l = const_cast<ID*>(l);
+
 		// Don't add locals that were already added because they're
 		// parameters.
 		//
@@ -203,11 +213,6 @@ StmtPtr ZAMCompiler::CompileBody()
 	// Could erase insts1 here to recover memory, but it's handy
 	// for debugging.
 
-#if 0
-	if ( non_recursive )
-		func->UseStaticFrame();
-#endif
-
 	auto zb = make_intrusive<ZBody>(func->Name(), this);
 	zb->SetInsts(insts2);
 
