@@ -24,6 +24,19 @@ bool ZAMCompiler::IsUnused(const IDPtr& id, const Stmt* where) const
 	return ! usage || ! usage->HasID(id.get());
 	}
 
+bool ZAMCompiler::IsCapture(const ID* id) const
+	{
+	const auto& c = pf->CapturesOffsets();
+	return c.find(id) != c.end();
+	}
+
+int ZAMCompiler::CaptureOffset(const ID* id) const
+	{
+	auto id_offset = pf->CapturesOffsets().find(id);
+	ASSERT(id_offset != pf->CapturesOffsets().end());
+	return id_offset->second;
+	}
+
 void ZAMCompiler::LoadParam(const ID* id)
 	{
 	if ( id->IsType() )
@@ -64,9 +77,26 @@ const ZAMStmt ZAMCompiler::LoadGlobal(const ID* id)
 
 	// We use the id_val for reporting used-but-not-set errors.
 	z.aux = new ZInstAux(0);
-	z.aux->id_val = id;
+	z.aux->id_val = {NewRef{}, const_cast<ID*>(id)};
 
-	return AddInst(z);
+	return AddInst(z, true);
+	}
+
+const ZAMStmt ZAMCompiler::LoadCapture(const ID* id)
+	{
+	ZOp op;
+
+	if ( ZVal::IsManagedType(id->GetType()) )
+		op = OP_LOAD_MANAGED_CAPTURE_VV;
+	else
+		op = OP_LOAD_CAPTURE_VV;
+
+	auto slot = RawSlot(id);
+
+	ZInstI z(op, slot, CaptureOffset(id));
+	z.op_type = OP_VV_I2;
+
+	return AddInst(z, true);
 	}
 
 int ZAMCompiler::AddToFrame(const ID* id)
@@ -81,38 +111,37 @@ int ZAMCompiler::FrameSlot(const ID* id)
 	auto slot = RawSlot(id);
 
 	if ( id->IsGlobal() )
-		(void)LoadGlobal(frame_denizens[slot]);
+		(void)LoadGlobal(id);
+
+	else if ( IsCapture(id) )
+		(void)LoadCapture(id);
 
 	return slot;
 	}
 
 int ZAMCompiler::Frame1Slot(const ID* id, ZAMOp1Flavor fl)
 	{
-	auto slot = RawSlot(id);
+	if ( fl == OP1_READ )
+		return FrameSlot(id);
 
-	switch ( fl )
-		{
-		case OP1_READ:
-			if ( id->IsGlobal() )
-				(void)LoadGlobal(frame_denizens[slot]);
-			break;
+	if ( fl == OP1_INTERNAL )
+		return RawSlot(id);
 
-		case OP1_WRITE:
-			if ( id->IsGlobal() )
-				pending_global_store = global_id_to_info[id];
-			break;
+	ASSERT(fl == OP1_WRITE || fl == OP1_READ_WRITE);
 
-		case OP1_READ_WRITE:
-			if ( id->IsGlobal() )
-				{
-				(void)LoadGlobal(frame_denizens[slot]);
-				pending_global_store = global_id_to_info[id];
-				}
-			break;
+	// Important: get the slot *before* tracking non-locals, so we don't
+	// prematurely generate a Store for the read/write case.
+	auto slot = fl == OP1_READ_WRITE ? FrameSlot(id) : RawSlot(id);
 
-		case OP1_INTERNAL:
-			break;
-		}
+	if ( id->IsGlobal() )
+		pending_global_store = global_id_to_info[id];
+
+	else if ( IsCapture(id) )
+		pending_capture_store = CaptureOffset(id);
+
+	// Make sure we don't think we're storing to both a global and
+	// a capture.
+	ASSERT(pending_global_store == -1 || pending_capture_store == -1);
 
 	return slot;
 	}
