@@ -33,7 +33,7 @@ public:
 	/**
 	 * Destructor.
 	 */
-	virtual ~Formatter();
+	virtual ~Formatter() = default;
 
 	/**
 	 * Convert a list of threading values into an implementation specific
@@ -62,7 +62,7 @@ public:
 	 *
 	 * @param val the Value to render to the ODesc object.
 	 *
-	 * @param The name of a field associated with the value.
+	 * @param name The name of a field associated with the value.
 	 *
 	 * @return Returns true on success, false on error. Errors are also
 	 * flagged via the thread.
@@ -75,7 +75,7 @@ public:
 	 *
 	 * @param s The string to parse.
 	 *
-	 * @param The name of a field associated with the value. Used only
+	 * @param name The name of a field associated with the value. Used only
 	 * for error reporting.
 	 *
 	 * @return The new value, or null on error. Errors must also be
@@ -107,7 +107,7 @@ public:
 	static std::string Render(const Value::subnet_t& subnet);
 
 	/**
-	 * Convert a double into a string. This renders the double with Bro's
+	 * Convert a double into a string. This renders the double with Zeek's
 	 * standard precision.
 	 *
 	 * This is a helper function that formatter implementations may use.
